#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Creates bands based on Fibonacci numbers and the EMA.
    /// </summary>
    [Description("Creates bands based on Fibonacci numbers and the EMA.")]
    public class FibonacciBands : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int length = 21; // Default setting for Length
			private double firstFactor = 1.618;
			private double secondFactor = 2.618;
			private double thirdFactor = 4.236;
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {

			
			Add(new Plot(Color.Black, PlotStyle.Line, "Midline"));		
			Add(new Plot(Color.Lime, PlotStyle.Line, "Fibo1"));
			Add(new Plot(Color.Red, PlotStyle.Line, "Fibo4"));				
			Add(new Plot(Color.Green, PlotStyle.Line, "Fibo2"));
			Add(new Plot(Color.ForestGreen, PlotStyle.Line, "Fibo3"));
			Add(new Plot(Color.Firebrick, PlotStyle.Line, "Fibo5"));			
			Add(new Plot(Color.SaddleBrown, PlotStyle.Line, "Fibo6"));			

            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            // Use this method for calculating your indicator values. Assign a value to each
            // plot below by replacing 'Close[0]' with your own formula.

			//Double tr = Math.Max(Close[1], High[0]) - Math.Min(Close[1],Low[0]);
			Double atr = ATR(length).Value[0];
			Double ema = EMA(Close, length).Value[0];

			Fibo3.Set(ema + atr * thirdFactor);
			Fibo2.Set(ema + atr * secondFactor);
			Fibo1.Set(ema + atr * firstFactor);
            Midline.Set(ema);
            Fibo4.Set(ema - atr * firstFactor);
			Fibo5.Set(ema - atr * secondFactor);
			Fibo6.Set(ema - atr * thirdFactor);
			
/*
			def tr = max(close[1], high) - min(close[1], low);
			def atr = average(tr,length);
			def ema = ExpAverage(close, length);
			
			plot fib1 = ema + atr * 1.618;
			plot fib2 = ema + atr * 2.618;
			plot fib3 = ema + atr * 4.236;
			plot ema1 = ema;
			plot fib4 = ema - atr * 1.618;
			plot fib5 = ema - atr * 2.618;
			plot fib6 = ema - atr * 4.236; 
			
			How to trade
			Place on two different time frames eg. 15 and 55 min. 
			Take trades off either short or long term chart.
			Best trades occur when both charts show same trigger/condition.
			Trades are short term reversals in direction of major trend on longer term chart unless you expect a trend reversal.
			Determine which band is the limiting band for the volatility of the instrument.
			When the market closes outside of the limiting band then returns inside, take a long/short one tick above/below
			the high/low of the previous bar. 
			Place stop below/above the low/high of the the recent swing low/high.
			Set targets at opposite band of chart with trailing stop after crossing middle band.
*/


        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Midline
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fibo1
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fibo2
        {
            get { return Values[3]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fibo3
        {
            get { return Values[4]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fibo4
        {
            get { return Values[2]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fibo5
        {
            get { return Values[5]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fibo6
        {
            get { return Values[6]; }
        }

        [Description("Length of EMA")]
        [Category("Parameters")]
        public int Length
        {
            get { return length; }
            set { length = Math.Max(1, value); }
        }
		
		[Description("First Factor")]
		[Category("Parameters")]
		public double FirstFactor
		{
			get { return firstFactor; }
			set { firstFactor = Math.Max(1.0, value); }
		}
			
		[Description("Second Factor")]
		[Category("Parameters")]
		public double SecondFactor
		{
			get { return secondFactor; }
			set { secondFactor = Math.Max(1.0, value); }
		}
				
		[Description("Third Factor")]
		[Category("Parameters")]
		public double ThirdFactor
		{
			get { return thirdFactor; }
			set { thirdFactor = Math.Max(1.0, value); }
		}
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private FibonacciBands[] cacheFibonacciBands = null;

        private static FibonacciBands checkFibonacciBands = new FibonacciBands();

        /// <summary>
        /// Creates bands based on Fibonacci numbers and the EMA.
        /// </summary>
        /// <returns></returns>
        public FibonacciBands FibonacciBands(double firstFactor, int length, double secondFactor, double thirdFactor)
        {
            return FibonacciBands(Input, firstFactor, length, secondFactor, thirdFactor);
        }

        /// <summary>
        /// Creates bands based on Fibonacci numbers and the EMA.
        /// </summary>
        /// <returns></returns>
        public FibonacciBands FibonacciBands(Data.IDataSeries input, double firstFactor, int length, double secondFactor, double thirdFactor)
        {
            checkFibonacciBands.FirstFactor = firstFactor;
            firstFactor = checkFibonacciBands.FirstFactor;
            checkFibonacciBands.Length = length;
            length = checkFibonacciBands.Length;
            checkFibonacciBands.SecondFactor = secondFactor;
            secondFactor = checkFibonacciBands.SecondFactor;
            checkFibonacciBands.ThirdFactor = thirdFactor;
            thirdFactor = checkFibonacciBands.ThirdFactor;

            if (cacheFibonacciBands != null)
                for (int idx = 0; idx < cacheFibonacciBands.Length; idx++)
                    if (Math.Abs(cacheFibonacciBands[idx].FirstFactor - firstFactor) <= double.Epsilon && cacheFibonacciBands[idx].Length == length && Math.Abs(cacheFibonacciBands[idx].SecondFactor - secondFactor) <= double.Epsilon && Math.Abs(cacheFibonacciBands[idx].ThirdFactor - thirdFactor) <= double.Epsilon && cacheFibonacciBands[idx].EqualsInput(input))
                        return cacheFibonacciBands[idx];

            FibonacciBands indicator = new FibonacciBands();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.FirstFactor = firstFactor;
            indicator.Length = length;
            indicator.SecondFactor = secondFactor;
            indicator.ThirdFactor = thirdFactor;
            indicator.SetUp();

            FibonacciBands[] tmp = new FibonacciBands[cacheFibonacciBands == null ? 1 : cacheFibonacciBands.Length + 1];
            if (cacheFibonacciBands != null)
                cacheFibonacciBands.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheFibonacciBands = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Creates bands based on Fibonacci numbers and the EMA.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FibonacciBands FibonacciBands(double firstFactor, int length, double secondFactor, double thirdFactor)
        {
            return _indicator.FibonacciBands(Input, firstFactor, length, secondFactor, thirdFactor);
        }

        /// <summary>
        /// Creates bands based on Fibonacci numbers and the EMA.
        /// </summary>
        /// <returns></returns>
        public Indicator.FibonacciBands FibonacciBands(Data.IDataSeries input, double firstFactor, int length, double secondFactor, double thirdFactor)
        {
            return _indicator.FibonacciBands(input, firstFactor, length, secondFactor, thirdFactor);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Creates bands based on Fibonacci numbers and the EMA.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FibonacciBands FibonacciBands(double firstFactor, int length, double secondFactor, double thirdFactor)
        {
            return _indicator.FibonacciBands(Input, firstFactor, length, secondFactor, thirdFactor);
        }

        /// <summary>
        /// Creates bands based on Fibonacci numbers and the EMA.
        /// </summary>
        /// <returns></returns>
        public Indicator.FibonacciBands FibonacciBands(Data.IDataSeries input, double firstFactor, int length, double secondFactor, double thirdFactor)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.FibonacciBands(input, firstFactor, length, secondFactor, thirdFactor);
        }

    }
}
#endregion
